"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const exec_time_1 = require("./exec-time");
const test_utils_1 = require("../common/test-utils");
describe('exec-time', () => {
    it('should make sure error thrown when decorator not set on method', () => {
        try {
            const nonValidExecTime = (0, exec_time_1.execTime)();
            class T {
            }
            __decorate([
                nonValidExecTime,
                __metadata("design:type", String)
            ], T.prototype, "boo", void 0);
        }
        catch (e) {
            expect('@execTime is applicable only on a methods.').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should make sure that the reporter is called with the correct data when decorated method is sync', () => {
        const reporter = jest.fn();
        class T {
            foo(x) {
                return `${x}b`;
            }
        }
        __decorate([
            (0, exec_time_1.execTime)(reporter),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String]),
            __metadata("design:returntype", String)
        ], T.prototype, "foo", null);
        const t = new T();
        t.foo('a');
        expect(reporter).toBeCalledTimes(1);
        const args = reporter.mock.calls[0][0];
        expect(args.args).toEqual(['a']);
        expect(args.result).toEqual('ab');
        expect(args.execTime).toBeGreaterThanOrEqual(0);
        expect(args.execTime).toBeLessThan(10);
    });
    it('should make sure that the reporter is called with the correct data when decorated method is async', () => __awaiter(void 0, void 0, void 0, function* () {
        const reporter = jest.fn();
        class T {
            foo(x) {
                return __awaiter(this, void 0, void 0, function* () {
                    yield (0, test_utils_1.sleep)(10);
                    return Promise.resolve(`${x}b`);
                });
            }
        }
        __decorate([
            (0, exec_time_1.execTime)(reporter),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        yield t.foo('a');
        expect(reporter).toBeCalledTimes(1);
        const args = reporter.mock.calls[0][0];
        expect(args.args).toEqual(['a']);
        expect(args.result).toEqual('ab');
        expect(args.execTime).toBeGreaterThanOrEqual(8);
        expect(args.execTime).toBeLessThan(20);
    }));
    it('should make sure that the console.log is being called by default', () => __awaiter(void 0, void 0, void 0, function* () {
        const logSpy = jest.spyOn(global.console, 'info');
        class T {
            foo(x) {
                return `${x}b`;
            }
        }
        __decorate([
            (0, exec_time_1.execTime)(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String]),
            __metadata("design:returntype", String)
        ], T.prototype, "foo", null);
        const t = new T();
        yield t.foo('a');
        expect(logSpy).toBeCalledTimes(1);
        const clogSpyArgs = logSpy.mock.calls[0][0];
        expect(clogSpyArgs).toBeGreaterThanOrEqual(0);
        logSpy.mockRestore();
    }));
    it('should make sure that the reporter is called when provided as sting', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.goo = jest.fn();
            }
            foo(x) {
                return __awaiter(this, void 0, void 0, function* () {
                    yield (0, test_utils_1.sleep)(10);
                    return Promise.resolve(`${x}b`);
                });
            }
        }
        __decorate([
            (0, exec_time_1.execTime)('goo'),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        yield t.foo('a');
        expect(t.goo).toBeCalledTimes(1);
        const args = t.goo.mock.calls[0][0];
        expect(args.args).toEqual(['a']);
        expect(args.result).toEqual('ab');
        expect(args.execTime).toBeGreaterThanOrEqual(8);
        expect(args.execTime).toBeLessThan(20);
    }));
});
//# sourceMappingURL=exec-time.spec.js.map