var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { debounce } from './debounce';
import { sleep } from '../common/test-utils';
describe('debounce', () => {
    it('should make sure error thrown when decorator not set on method', () => {
        try {
            const nonValidDebounce = debounce(50);
            class T {
            }
            __decorate([
                nonValidDebounce,
                __metadata("design:type", String)
            ], T.prototype, "boo", void 0);
        }
        catch (e) {
            expect('@debounce is applicable only on a methods.').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should verify method invocation is debounced', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            foo() {
                return this.goo();
            }
            goo() {
                expect(this.prop).toBe(3);
            }
        }
        __decorate([
            debounce(30),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", void 0)
        ], T.prototype, "foo", null);
        const t = new T();
        t.prop = 3;
        const spy = jest.spyOn(T.prototype, 'goo');
        t.foo(); // 15
        expect(spy).not.toBeCalled();
        yield sleep(10);
        expect(spy).toBeCalledTimes(0);
        t.foo(); // 20
        yield sleep(20);
        expect(spy).toBeCalledTimes(0);
        yield sleep(40);
        expect(spy).toBeCalledTimes(1);
    }));
    it('should verify method params are passed', (done) => {
        class T {
            foo(x, y) {
                return this.goo(x, y);
            }
            goo(x, y) {
            }
        }
        __decorate([
            debounce(5),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number, Number]),
            __metadata("design:returntype", void 0)
        ], T.prototype, "foo", null);
        const t = new T();
        const spy = jest.spyOn(T.prototype, 'goo');
        t.foo(1, 2);
        setTimeout(() => {
            expect(spy).toHaveBeenCalledWith(1, 2);
            done();
        }, 10);
    });
});
//# sourceMappingURL=debounce.spec.js.map