var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { delegate } from './delegate';
import { sleep } from '../common/test-utils';
describe('delegate', () => {
    it('should make sure error thrown when decorator not set on method', () => {
        try {
            const nonValidDelegate = delegate();
            class T {
            }
            __decorate([
                nonValidDelegate,
                __metadata("design:type", String)
            ], T.prototype, "boo", void 0);
        }
        catch (e) {
            expect('@delegate is applicable only on a methods.').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should delegate method with same key invocation', () => __awaiter(void 0, void 0, void 0, function* () {
        let counter = 0;
        class T {
            foo() {
                return __awaiter(this, void 0, void 0, function* () {
                    counter += 1;
                    yield sleep(20);
                    return Promise.resolve(1);
                });
            }
        }
        __decorate([
            delegate(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        t.foo();
        t.foo();
        const res = yield Promise.all([t.foo(), t.foo()]);
        expect(res).toEqual([1, 1]);
        expect(counter).toEqual(1);
    }));
    it('should delegate method with same key invocation until delegator is resolved / rejected', () => __awaiter(void 0, void 0, void 0, function* () {
        const timestampBeforeTest = Date.now();
        let counter = 0;
        class T {
            foo() {
                return __awaiter(this, void 0, void 0, function* () {
                    counter += 1;
                    yield sleep(20);
                    return Date.now();
                });
            }
        }
        __decorate([
            delegate(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        t.foo();
        t.foo();
        const res0 = yield Promise.all([t.foo(), t.foo()]);
        expect(res0[0]).toEqual(res0[1]);
        expect(res0[0]).toBeGreaterThan(timestampBeforeTest);
        expect(counter).toEqual(1);
        t.foo();
        t.foo();
        const res1 = yield Promise.all([t.foo(), t.foo()]);
        expect(res1).not.toEqual(res0);
        expect(res1[0]).toEqual(res1[1]);
        expect(res1[0]).toBeGreaterThan(res0[0]);
        expect(counter).toEqual(2);
    }));
    it('should delegate method with same key invocation - default key serialization', () => __awaiter(void 0, void 0, void 0, function* () {
        let counter = 0;
        class T {
            foo(x) {
                return __awaiter(this, void 0, void 0, function* () {
                    counter += 1;
                    yield sleep(20);
                    return Promise.resolve(x);
                });
            }
        }
        __decorate([
            delegate(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        const res = yield Promise.all([t.foo('a'), t.foo('a'), t.foo('b')]);
        expect(res).toEqual(['a', 'a', 'b']);
        expect(counter).toEqual(2);
    }));
    it('should delegate method with same key invocation - custom serialization', () => __awaiter(void 0, void 0, void 0, function* () {
        let counter = 0;
        class T {
            foo(a, b) {
                return __awaiter(this, void 0, void 0, function* () {
                    counter += 1;
                    yield sleep(20);
                    return Promise.resolve(a + b);
                });
            }
        }
        __decorate([
            delegate((a, b) => `${a}_${b}`),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number, Number]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        const res = yield Promise.all([t.foo(1, 1), t.foo(2, 1), t.foo(1, 1)]);
        expect(res).toEqual([2, 3, 2]);
        expect(counter).toEqual(2);
    }));
    it('should have the correct context', () => __awaiter(void 0, void 0, void 0, function* () {
        class Example {
            static ex2() {
                return Promise.resolve(2);
            }
            static ex1() {
                return __awaiter(this, void 0, void 0, function* () {
                    return Example.ex2();
                });
            }
        }
        __decorate([
            delegate(),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], Example, "ex1", null);
        const result = yield Example.ex1();
        expect(result).toEqual(2);
    }));
});
//# sourceMappingURL=delegate.spec.js.map