var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { memoizeAsync } from './memoize-async';
import { sleep } from '../common/test-utils';
describe('memozie-async', () => {
    it('should verify memoize async caching original method', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            foo(x, y) {
                return this.goo(x, y);
            }
            goo(x, y) {
                expect(this.prop).toBe(3);
                return Promise.resolve(x + y);
            }
        }
        __decorate([
            memoizeAsync(10),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Number, Number]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        return new Promise((res) => {
            const t = new T();
            t.prop = 3;
            const spy = jest.spyOn(T.prototype, 'goo');
            const resp1 = t.foo(1, 2);
            const resp2 = t.foo(1, 2);
            setTimeout(() => {
                expect(spy).toHaveBeenCalledWith(1, 2);
                expect(spy).toBeCalledTimes(1);
                const resp01 = t.foo(1, 3);
                setTimeout(() => {
                    expect(spy).toHaveBeenCalledWith(1, 3);
                    expect(spy).toBeCalledTimes(2);
                }, 0);
                setTimeout(() => __awaiter(void 0, void 0, void 0, function* () {
                    const resp3 = t.foo(1, 2);
                    setTimeout(() => __awaiter(void 0, void 0, void 0, function* () {
                        expect(spy).toHaveBeenCalledWith(1, 2);
                        expect(spy).toBeCalledTimes(3);
                        expect(yield resp1).toBe(3);
                        expect(yield resp2).toBe(3);
                        expect(yield resp3).toBe(3);
                        expect(yield resp01).toBe(4);
                        res(null);
                    }), 0);
                }), 20);
            }, 0);
        });
    }));
    it('should verify memoize key foo', () => __awaiter(void 0, void 0, void 0, function* () {
        const mapper = jest.fn((x, y) => `${x}_${y}`);
        class T {
            fooWithMapper(x, y) {
                return this.goo(x, y);
            }
            goo(x, y) {
                return Promise.resolve(x + y);
            }
        }
        __decorate([
            memoizeAsync({ expirationTimeMs: 10, keyResolver: mapper }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String, String]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "fooWithMapper", null);
        return new Promise((resolve) => {
            const t = new T();
            const spyFooWithMapper = jest.spyOn(T.prototype, 'goo');
            t.fooWithMapper('x', 'y');
            t.fooWithMapper('x', 'y');
            setTimeout(() => {
                expect(mapper.mock.calls.length).toBe(2);
                expect(spyFooWithMapper).toHaveBeenCalledTimes(1);
                expect(spyFooWithMapper).toHaveBeenCalledWith('x', 'y');
                resolve(null);
            }, 0);
        });
    }));
    it('should verify memoize key foo as string - target method', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            foo(x, y) {
                return `${x}_${y}`;
            }
            fooWithMapper(x, y) {
                return this.goo(x, y);
            }
            goo(x, y) {
                return Promise.resolve(x + y);
            }
        }
        __decorate([
            memoizeAsync({ expirationTimeMs: 10, keyResolver: 'foo' }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String, String]),
            __metadata("design:returntype", Promise)
        ], T.prototype, "fooWithMapper", null);
        return new Promise((resolve) => {
            const t = new T();
            const spyFooWithMapper = jest.spyOn(T.prototype, 'goo');
            const mapper = jest.spyOn(T.prototype, 'foo');
            t.fooWithMapper('x', 'y');
            t.fooWithMapper('x', 'y');
            setTimeout(() => {
                expect(mapper).toHaveBeenCalledTimes(2);
                expect(spyFooWithMapper).toHaveBeenCalledTimes(1);
                expect(spyFooWithMapper).toHaveBeenCalledWith('x', 'y');
                expect(mapper).toHaveBeenCalledWith('x', 'y');
                resolve(null);
            }, 0);
        });
    }));
    it('should make sure error thrown when decorator not set on method', () => {
        try {
            const nonValidMemoizeAsync = memoizeAsync(50);
            class T {
            }
            __decorate([
                nonValidMemoizeAsync,
                __metadata("design:type", String)
            ], T.prototype, "boo", void 0);
        }
        catch (e) {
            expect('@memoizeAsync is applicable only on a methods.').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should make sure that when promise is rejected it is removed from the cache', (done) => {
        class T {
            foo() {
                return this.goo();
            }
            goo() {
                return Promise.reject(new Error('rejected'));
            }
        }
        __decorate([
            memoizeAsync(20),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        const spy = jest.spyOn(T.prototype, 'goo');
        t.foo()
            .catch((e) => {
            expect(e.message).toBe('rejected');
        });
        setTimeout(() => {
            t.foo().catch((e) => {
                expect(e.message).toBe('rejected');
            });
            setTimeout(() => {
                expect(spy).toHaveBeenCalledTimes(2);
                done();
            }, 0);
        }, 20);
    });
    it('should use provided cache', (done) => {
        const cache = new Map();
        class T {
            foo() {
                return this.goo();
            }
            goo() {
                return Promise.resolve(1);
            }
        }
        __decorate([
            memoizeAsync({ expirationTimeMs: 30, cache }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const spy = jest.spyOn(T.prototype, 'goo');
        const t = new T();
        t.foo();
        setTimeout(() => {
            t.foo();
            setTimeout(() => {
                expect(spy).toHaveBeenCalledTimes(1);
                cache.delete('[]');
                t.foo();
                setTimeout(() => {
                    expect(spy).toHaveBeenCalledTimes(2);
                    done();
                }, 0);
            }, 0);
        }, 10);
    });
    it('should use different scope to different usages', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            foo() {
                return Promise.resolve(1);
            }
            goo() {
                return Promise.resolve(2);
            }
        }
        __decorate([
            memoizeAsync(20),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        __decorate([
            memoizeAsync(20),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "goo", null);
        const t = new T();
        const one = t.foo();
        const two = t.goo();
        expect(yield one).toBe(1);
        expect(yield two).toBe(2);
    }));
    it('should verify that by default the cache is never cleaned', () => __awaiter(void 0, void 0, void 0, function* () {
        const cache = new Map();
        class T {
            foo() {
                return Promise.resolve(1);
            }
        }
        __decorate([
            memoizeAsync({ cache }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        yield t.foo();
        expect(cache.size).toEqual(1);
        yield sleep(50);
        expect(cache.size).toEqual(1);
    }));
    it('should verify usage of async cache', () => __awaiter(void 0, void 0, void 0, function* () {
        const map = new Map();
        const cache = {
            delete: (p1) => __awaiter(void 0, void 0, void 0, function* () {
                map.delete(p1);
            }),
            get: (p1) => __awaiter(void 0, void 0, void 0, function* () { return map.get(p1); }),
            has: (p1) => __awaiter(void 0, void 0, void 0, function* () { return map.has(p1); }),
            set: (p1, p2) => __awaiter(void 0, void 0, void 0, function* () {
                map.set(p1, p2);
            }),
        };
        class T {
            foo() {
                return this.goo();
            }
            goo() {
                return Promise.resolve(1);
            }
        }
        __decorate([
            memoizeAsync({
                expirationTimeMs: 30,
                cache,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        return new Promise((resolve) => {
            const spy = jest.spyOn(T.prototype, 'goo');
            const t = new T();
            t.foo();
            setTimeout(() => {
                t.foo();
                setTimeout(() => {
                    expect(spy).toHaveBeenCalledTimes(1);
                    cache.delete('[]');
                    t.foo();
                    setTimeout(() => {
                        expect(spy).toHaveBeenCalledTimes(2);
                        resolve(null);
                    }, 0);
                }, 0);
            }, 10);
        });
    }));
    it('should throw exception when async has method throws an exception', () => __awaiter(void 0, void 0, void 0, function* () {
        const map = new Map();
        const cache = {
            delete: (p1) => __awaiter(void 0, void 0, void 0, function* () {
                map.delete(p1);
            }),
            get: (p1) => __awaiter(void 0, void 0, void 0, function* () { return map.get(p1); }),
            has: (p1) => __awaiter(void 0, void 0, void 0, function* () { return Promise.reject(new Error('error')); }),
            set: (p1, p2) => __awaiter(void 0, void 0, void 0, function* () {
                map.set(p1, p2);
            }),
        };
        class T {
            foo() {
                return Promise.resolve(1);
            }
        }
        __decorate([
            memoizeAsync({
                expirationTimeMs: 30,
                cache,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        try {
            yield t.foo();
        }
        catch (e) {
            expect(e.message).toBe('error');
            return;
        }
        throw new Error('shouldn\'t get to here');
    }));
    it('should throw exception when async get method throws an exception', () => __awaiter(void 0, void 0, void 0, function* () {
        const map = new Map();
        const cache = {
            delete: (p1) => __awaiter(void 0, void 0, void 0, function* () {
                map.delete(p1);
            }),
            get: (p1) => __awaiter(void 0, void 0, void 0, function* () { return Promise.reject(new Error('error')); }),
            has: (p1) => __awaiter(void 0, void 0, void 0, function* () { return Promise.resolve(true); }),
            set: (p1, p2) => __awaiter(void 0, void 0, void 0, function* () {
                map.set(p1, p2);
            }),
        };
        class T {
            foo() {
                return Promise.resolve(1);
            }
        }
        __decorate([
            memoizeAsync({
                expirationTimeMs: 30,
                cache,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        try {
            yield t.foo();
        }
        catch (e) {
            expect(e.message).toBe('error');
            return;
        }
        throw new Error('shouldn\'t get to here');
    }));
    it('should throw exception when async get method throws an exception', () => __awaiter(void 0, void 0, void 0, function* () {
        const map = new Map();
        const cache = {
            delete: (p1) => __awaiter(void 0, void 0, void 0, function* () {
                map.delete(p1);
            }),
            get: (p1) => __awaiter(void 0, void 0, void 0, function* () { return Promise.reject(new Error('error')); }),
            has: (p1) => __awaiter(void 0, void 0, void 0, function* () { return true; }),
            set: (p1, p2) => __awaiter(void 0, void 0, void 0, function* () {
                map.set(p1, p2);
            }),
        };
        class T {
            foo() {
                return Promise.resolve(1);
            }
        }
        __decorate([
            memoizeAsync({
                expirationTimeMs: 30,
                cache,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        try {
            yield t.foo();
        }
        catch (e) {
            expect(e.message).toBe('error');
            return;
        }
        throw new Error('shouldn\'t get to here');
    }));
    it('should throw exception when original method is broken', () => __awaiter(void 0, void 0, void 0, function* () {
        const map = new Map();
        const cache = {
            delete: (p1) => __awaiter(void 0, void 0, void 0, function* () {
                map.delete(p1);
            }),
            get: (p1) => __awaiter(void 0, void 0, void 0, function* () { return map.get(p1); }),
            has: (p1) => __awaiter(void 0, void 0, void 0, function* () { return map.has(p1); }),
            set: (p1, p2) => __awaiter(void 0, void 0, void 0, function* () {
                map.set(p1, p2);
            }),
        };
        class T {
            foo() {
                return Promise.reject(new Error('error'));
            }
        }
        __decorate([
            memoizeAsync({
                expirationTimeMs: 30,
                cache,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        try {
            yield t.foo();
        }
        catch (e) {
            expect(e.message).toBe('error');
            return;
        }
        throw new Error('shouldn\'t get to here');
    }));
});
//# sourceMappingURL=memoize-async.spec.js.map