var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { rateLimit } from './rate-limit';
import { sleep } from '../common/test-utils';
import { SimpleRateLimitCounter } from './simple-rate-limit-counter';
export class AsyncSimpleRateLimitCounter {
    constructor() {
        this.counterMap = new Map();
    }
    getCount(key) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            yield sleep(10);
            return (_a = this.counterMap.get(key)) !== null && _a !== void 0 ? _a : 0;
        });
    }
    inc(key) {
        return __awaiter(this, void 0, void 0, function* () {
            yield sleep(10);
            if (!this.counterMap.has(key)) {
                yield sleep(10);
                this.counterMap.set(key, 0);
            }
            yield sleep(10);
            this.counterMap.set(key, this.counterMap.get(key) + 1);
        });
    }
    dec(key) {
        return __awaiter(this, void 0, void 0, function* () {
            yield sleep(10);
            const currentCount = this.counterMap.get(key);
            yield sleep(10);
            if (currentCount === 1) {
                this.counterMap.delete(key);
            }
            else {
                this.counterMap.set(key, currentCount - 1);
            }
        });
    }
}
describe('rate-limit', () => {
    class T {
        constructor() {
            this.counter = 0;
        }
        foo() {
            this.counter += 1;
            return this.counter;
        }
    }
    __decorate([
        rateLimit({ allowedCalls: 2, timeSpanMs: 200 }),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], T.prototype, "foo", null);
    it('should make sure error thrown when decorator not set on method', () => {
        try {
            const nonValidRateLimit = rateLimit({
                allowedCalls: 1,
                timeSpanMs: 1000,
            });
            class TT {
            }
            __decorate([
                nonValidRateLimit,
                __metadata("design:type", String)
            ], TT.prototype, "boo", void 0);
        }
        catch (e) {
            expect('@rateLimit is applicable only on a method.').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should make sure error on async and sync counters', () => {
        try {
            class TT {
                foo() {
                }
            }
            __decorate([
                rateLimit({
                    allowedCalls: 1,
                    timeSpanMs: 1000,
                    rateLimitAsyncCounter: {},
                    rateLimitCounter: {},
                }),
                __metadata("design:type", Function),
                __metadata("design:paramtypes", []),
                __metadata("design:returntype", void 0)
            ], TT.prototype, "foo", null);
            throw new Error('should not reach this line');
        }
        catch (e) {
            expect('You cant provide both rateLimitAsyncCounter and rateLimitCounter.').toBe(e.message);
        }
    });
    it('should verify sync limit is working as expected', () => __awaiter(void 0, void 0, void 0, function* () {
        const t = new T();
        t.foo();
        yield sleep(50);
        expect(t.counter).toEqual(1);
        t.foo();
        yield sleep(50);
        expect(t.counter).toEqual(2);
        yield sleep(50);
        try {
            t.foo();
            throw new Error('should not get to this line');
        }
        catch (e) {
            expect(e.message).toEqual('You have acceded the amount of allowed calls');
        }
        yield sleep(80);
        const resp = t.foo();
        expect(t.counter).toEqual(3);
        expect(resp).toEqual(3);
    }));
    it('should verify sync limit is working as expected with custom Rate Limiter', () => __awaiter(void 0, void 0, void 0, function* () {
        const countMap = new Map();
        class TT {
            constructor() {
                this.counter = 0;
            }
            foo() {
                this.counter += 1;
            }
        }
        __decorate([
            rateLimit({
                allowedCalls: 2,
                timeSpanMs: 200,
                rateLimitCounter: new SimpleRateLimitCounter(countMap),
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", void 0)
        ], TT.prototype, "foo", null);
        const t = new TT();
        t.foo();
        expect(countMap.size).toEqual(1);
        yield sleep(50);
        expect(t.counter).toEqual(1);
        t.foo();
        expect(countMap.size).toEqual(1);
        expect(countMap.get('__rateLimit__')).toEqual(2);
        yield sleep(50);
        expect(t.counter).toEqual(2);
        yield sleep(50);
        try {
            t.foo();
            throw new Error('should not get to this line');
        }
        catch (e) {
            expect(e.message).toEqual('You have acceded the amount of allowed calls');
        }
        yield sleep(80);
        expect(countMap.get('__rateLimit__')).toEqual(1);
        t.foo();
        expect(countMap.get('__rateLimit__')).toEqual(2);
        expect(t.counter).toEqual(3);
        yield sleep(220);
        expect(countMap.size).toEqual(0);
    }));
    it('should verify async limit is working as expected with custom Rate Limiter', () => __awaiter(void 0, void 0, void 0, function* () {
        const counter = new AsyncSimpleRateLimitCounter();
        class TT {
            constructor() {
                this.counter = 0;
            }
            foo() {
                return __awaiter(this, void 0, void 0, function* () {
                    this.counter += 1;
                    return this.counter;
                });
            }
        }
        __decorate([
            rateLimit({
                allowedCalls: 2,
                timeSpanMs: 200,
                rateLimitAsyncCounter: counter,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], TT.prototype, "foo", null);
        const t1 = new TT();
        const t2 = new TT();
        yield t1.foo();
        yield sleep(50);
        expect(t1.counter).toEqual(1);
        yield t2.foo();
        yield sleep(50);
        expect(t1.counter).toEqual(1);
        expect(t2.counter).toEqual(1);
        yield sleep(50);
        try {
            yield t1.foo();
            throw new Error('should not get to this line');
        }
        catch (e) {
            expect(e.message).toEqual('You have acceded the amount of allowed calls');
        }
        yield sleep(80);
        const resp1 = yield t1.foo();
        const resp2 = yield t2.foo();
        expect(t1.counter).toEqual(2);
        expect(resp1).toEqual(2);
        expect(resp2).toEqual(2);
        expect(counter.counterMap.has('__rateLimit__')).toEqual(true);
    }));
    it('should invoke provider mapper provided as function', () => __awaiter(void 0, void 0, void 0, function* () {
        class TT {
            foo(x) {
            }
        }
        __decorate([
            rateLimit({
                allowedCalls: 1,
                timeSpanMs: 200,
                keyResolver: (x) => x,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String]),
            __metadata("design:returntype", void 0)
        ], TT.prototype, "foo", null);
        const t = new TT();
        t.foo('a');
        yield sleep(50);
        t.foo('b');
        yield sleep(50);
        try {
            t.foo('a');
            throw new Error('should not get to this line');
        }
        catch (e) {
            expect(e.message).toEqual('You have acceded the amount of allowed calls');
        }
        yield sleep(120);
        t.foo('a');
    }));
    it('should invoke provider mapper provided as string', () => __awaiter(void 0, void 0, void 0, function* () {
        class TT {
            foo(x) {
            }
            goo(x) {
                return x;
            }
        }
        __decorate([
            rateLimit({
                allowedCalls: 1,
                timeSpanMs: 200,
                keyResolver: 'goo',
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String]),
            __metadata("design:returntype", void 0)
        ], TT.prototype, "foo", null);
        const t = new TT();
        t.foo('a');
        yield sleep(50);
        t.foo('b');
        yield sleep(50);
        try {
            t.foo('a');
            throw new Error('should not get to this line');
        }
        catch (e) {
            expect(e.message).toEqual('You have acceded the amount of allowed calls');
        }
        yield sleep(120);
        t.foo('a');
    }));
    it('should invoke custom handler when exceeds the amount of allowed calls', () => __awaiter(void 0, void 0, void 0, function* () {
        class TT {
            foo() {
            }
        }
        __decorate([
            rateLimit({
                allowedCalls: 1,
                timeSpanMs: 50,
                exceedHandler: () => {
                    throw new Error('blarg');
                },
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", void 0)
        ], TT.prototype, "foo", null);
        const t = new TT();
        t.foo();
        yield sleep(20);
        try {
            t.foo();
            throw new Error('should not get to this line');
        }
        catch (e) {
            expect(e.message).toEqual('blarg');
        }
    }));
});
//# sourceMappingURL=rate-limit.spec.js.map