var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { retry } from './retry';
import { sleep } from '../common/test-utils';
import { retryfy } from './retryfy';
describe('retry', () => {
    it('should make sure error thrown when decorator not set on method', () => {
        try {
            const nonValidRetry = retry(50);
            class T {
            }
            __decorate([
                nonValidRetry,
                __metadata("design:type", String)
            ], T.prototype, "boo", void 0);
        }
        catch (e) {
            expect('@retry is applicable only on a methods.').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should retry twice', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.counter = 0;
            }
            foo() {
                if (this.counter === 0) {
                    this.counter += 1;
                    return Promise.reject(new Error('no'));
                }
                return Promise.resolve('yes');
            }
        }
        __decorate([
            retry({
                retries: 2,
                delay: 10,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        const res = yield t.foo();
        expect(t.counter).toEqual(1);
        expect(res).toEqual('yes');
    }));
    it('should throw exception after retries', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.counter = 0;
            }
            foo() {
                this.counter += 1;
                return Promise.reject(new Error('no'));
            }
        }
        __decorate([
            retry({
                retries: 2,
                delay: 10,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        try {
            yield t.foo();
            throw new Error('should not get hear');
        }
        catch (e) {
            expect(t.counter).toEqual(3);
            expect(e.message).toEqual('no');
        }
    }));
    it('should wait according to configured delay', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.counter = 0;
            }
            foo() {
                this.counter += 1;
                if (this.counter < 3) {
                    return Promise.reject(new Error('no'));
                }
                return Promise.resolve('yes');
            }
        }
        __decorate([
            retry({
                retries: 2,
                delay: 50,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        t.foo();
        yield sleep(25);
        expect(t.counter).toEqual(1);
        yield sleep(50);
        expect(t.counter).toEqual(2);
        yield sleep(75);
        expect(t.counter).toEqual(3);
    }));
    it('should wait according to retries array', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.counter = 0;
            }
            foo() {
                this.counter += 1;
                if (this.counter !== 3) {
                    return Promise.reject(new Error('no'));
                }
                return Promise.resolve('yes');
            }
        }
        __decorate([
            retry([50, 100]),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        t.foo();
        yield sleep(25);
        expect(t.counter).toEqual(1);
        yield sleep(50);
        expect(t.counter).toEqual(2);
        yield sleep(150);
        expect(t.counter).toEqual(3);
    }));
    it('should wait 1 sec by default', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.counter = 0;
            }
            foo() {
                this.counter += 1;
                if (this.counter === 1) {
                    return Promise.reject(new Error('no'));
                }
                return Promise.resolve('yes');
            }
        }
        __decorate([
            retry(1),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        const prom = t.foo();
        yield sleep(500);
        expect(t.counter).toEqual(1);
        yield sleep(600);
        expect(t.counter).toEqual(2);
        expect(yield prom).toEqual('yes');
        expect(t.counter).toEqual(2);
    }));
    it('should throw error when invalid input', () => __awaiter(void 0, void 0, void 0, function* () {
        expect(retryfy.bind(this, '6')).toThrow('invalid input');
    }));
    it('should invoke onRetry', () => __awaiter(void 0, void 0, void 0, function* () {
        const onRetry = jest.fn();
        class T {
            constructor() {
                this.counter = 0;
                this.decCounter = 0;
            }
            foo() {
                this.counter += 1;
                if (this.counter < 3) {
                    return Promise.reject(new Error(`no ${this.counter}`));
                }
                return Promise.resolve('yes');
            }
            goo() {
                if (this.decCounter < 3) {
                    return Promise.reject(new Error(`no ${this.decCounter}`));
                }
                return Promise.resolve();
            }
            retry(e, c) {
                this.decCounter += 1;
            }
        }
        __decorate([
            retry({
                retries: 3,
                delay: 10,
                onRetry,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        __decorate([
            retry({
                retries: 3,
                delay: 10,
                onRetry: 'retry',
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "goo", null);
        const t = new T();
        yield t.foo();
        yield t.goo();
        yield sleep(100);
        expect(onRetry).toBeCalledTimes(2);
        const argsFirstCall = onRetry.mock.calls[0];
        expect(argsFirstCall[0].message).toEqual('no 1');
        expect(argsFirstCall[1]).toEqual(0);
        const argsSecondCall = onRetry.mock.calls[1];
        expect(argsSecondCall[0].message).toEqual('no 2');
        expect(argsSecondCall[1]).toEqual(1);
        expect(t.decCounter).toEqual(3);
    }));
    it('should throw error when provided both retires and delaysArray', () => {
        try {
            class T {
                boo() {
                    return Promise.resolve();
                }
            }
            __decorate([
                retry({
                    retries: 3,
                    delaysArray: [1, 2, 3],
                }),
                __metadata("design:type", Function),
                __metadata("design:paramtypes", []),
                __metadata("design:returntype", Promise)
            ], T.prototype, "boo", null);
        }
        catch (e) {
            expect('You can not provide both retries and delaysArray').toBe(e.message);
            return;
        }
        throw new Error('should not reach this line');
    });
    it('should fill the delays with 1000ms by default', () => __awaiter(void 0, void 0, void 0, function* () {
        class T {
            constructor() {
                this.counter = 0;
                this.decCounter = 0;
            }
            foo() {
                this.counter += 1;
                if (this.counter < 2) {
                    return Promise.reject(new Error(`no ${this.counter}`));
                }
                return Promise.resolve('yes');
            }
        }
        __decorate([
            retry({
                retries: 1,
            }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], T.prototype, "foo", null);
        const t = new T();
        t.foo();
        yield sleep(500);
        expect(t.counter).toEqual(1);
        yield sleep(600);
        expect(t.counter).toEqual(2);
    }));
});
//# sourceMappingURL=retry.spec.js.map